/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#ifndef _plstr_h^M
#define _plstr_h^M
^M
/*^M
 * plstr.h^M
 *^M
 * This header file exports the API to the NSPR portable library or string-^M
 * handling functions.  ^M
 * ^M
 * This API was not designed as an "optimal" or "ideal" string library; it ^M
 * was based on the good ol' unix string.3 functions, and was written to^M
 *^M
 *  1) replace the libc functions, for cross-platform consistancy, ^M
 *  2) complete the API on platforms lacking common functions (e.g., ^M
 *     strcase*), and^M
 *  3) to implement some obvious "closure" functions that I've seen^M
 *     people hacking around in our code.^M
 *^M
 * Point number three largely means that most functions have an "strn"^M
 * limited-length version, and all comparison routines have a non-case-^M
 * sensitive version available.^M
 */^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
/*^M
 * PL_strlen^M
 *^M
 * Returns the length of the provided string, not including the trailing '\0'.^M
 */^M
^M
PR_EXTERN(PRUint32)^M
PL_strlen(const char *str);^M
^M
/*^M
 * PL_strnlen^M
 *^M
 * Returns the length of the provided string, not including the trailing '\0',^M
 * up to the indicated maximum.  The string will not be examined beyond the^M
 * maximum; if no terminating '\0' is found, the maximum will be returned.^M
 */^M
^M
PR_EXTERN(PRUint32)^M
PL_strnlen(const char *str, PRUint32 max);^M
^M
/*^M
 * PL_strcpy^M
 *^M
 * Copies the source string, up to and including the trailing '\0', into the^M
 * destination buffer.  It does not (can not) verify that the destination^M
 * buffer is large enough.  It returns the "dest" argument.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strcpy(char *dest, const char *src);^M
^M
/*^M
 * PL_strncpy^M
 *^M
 * Copies the source string into the destination buffer, up to and including^M
 * the trailing '\0' or up to and including the max'th character, whichever^M
 * comes first.  It does not (can not) verify that the destination buffer is^M
 * large enough.  If the source string is longer than the maximum length,^M
 * the result will *not* be null-terminated (JLRU).^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strncpy(char *dest, const char *src, PRUint32 max);^M
^M
/*^M
 * PL_strncpyz^M
 *^M
 * Copies the source string into the destination buffer, up to and including ^M
 * the trailing '\0' or up but not including the max'th character, whichever ^M
 * comes first.  It does not (can not) verify that the destination buffer is^M
 * large enough.  The destination string is always terminated with a '\0',^M
 * unlike the traditional libc implementation.  It returns the "dest" argument.^M
 *^M
 * NOTE: If you call this with a source "abcdefg" and a max of 5, the ^M
 * destination will end up with "abcd\0" (i.e., it's strlen length will be 4)!^M
 *^M
 * This means you can do this:^M
 *^M
 *     char buffer[ SOME_SIZE ];^M
 *     PL_strncpyz(buffer, src, sizeof(buffer));^M
 *^M
 * and the result will be properly terminated.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strncpyz(char *dest, const char *src, PRUint32 max);^M
^M
/*^M
 * PL_strdup^M
 *^M
 * Returns a pointer to a malloc'd extent of memory containing a duplicate^M
 * of the argument string.  The size of the allocated extent is one greater^M
 * than the length of the argument string, because of the terminator.  A^M
 * null argument, like a zero-length argument, will result in a pointer to^M
 * a one-byte extent containing the null value.  This routine returns null^M
 * upon malloc failure.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strdup(const char *s);^M
^M
/*^M
 * PL_strfree^M
 *^M
 * Free memory allocated by PL_strdup^M
 */^M
^M
PR_EXTERN(void)^M
PL_strfree(char *s);^M
^M
/*^M
 * PL_strndup^M
 *^M
 * Returns a pointer to a malloc'd extent of memory containing a duplicate^M
 * of the argument string, up to the maximum specified.  If the argument^M
 * string has a length greater than the value of the specified maximum, the^M
 * return value will be a pointer to an extent of memory of length one^M
 * greater than the maximum specified.  A null string, a zero-length string,^M
 * or a zero maximum will all result in a pointer to a one-byte extent^M
 * containing the null value.  This routine returns null upon malloc failure.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strndup(const char *s, PRUint32 max);^M
^M
/*^M
 * PL_strcat^M
 *^M
 * Appends a copy of the string pointed to by the second argument to the^M
 * end of the string pointed to by the first.  The destination buffer is^M
 * not (can not be) checked for sufficient size.  A null destination^M
 * argument returns null; otherwise, the first argument is returned.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strcat(char *dst, const char *src);^M
^M
/*^M
 * PL_strncat^M
 *^M
 * Appends a copy of the string pointed to by the second argument, up to^M
 * the maximum size specified, to the end of the string pointed to by the^M
 * first.  The destination buffer is not (can not be) checked for sufficient^M
 * size.  A null destination argument returns null; otherwise, the first ^M
 * argument is returned.  If the maximum size limits the copy, then the^M
 * result will *not* be null-terminated (JLRU).  A null destination^M
 * returns null; otherwise, the destination argument is returned.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strncat(char *dst, const char *src, PRUint32 max);^M
^M
/*^M
 * PL_strcatn^M
 *^M
 * Appends a copy of the string pointed to by the third argument, to the^M
 * end of the string pointed to by the first.  The second argument specifies^M
 * the maximum size of the destination buffer, including the null termination.^M
 * If the existing string in dst is longer than the max, no action is taken.^M
 * The resulting string will be null-terminated.  A null destination returns^M
 * null; otherwise, the destination argument is returned.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strcatn(char *dst, PRUint32 max, const char *src);^M
^M
/*^M
 * PL_strcmp^M
 *^M
 * Returns an integer, the sign of which -- positive, zero, or negative --^M
 * reflects the lexical sorting order of the two strings indicated.  The^M
 * result is positive if the first string comes after the second.  The^M
 * NSPR implementation is not i18n.^M
 */^M
^M
PR_EXTERN(PRIntn)^M
PL_strcmp(const char *a, const char *b);^M
^M
/*^M
 * PL_strncmp^M
 * ^M
 * Returns an integer, the sign of which -- positive, zero, or negative --^M
 * reflects the lexical sorting order of the two strings indicated, up to^M
 * the maximum specified.  The result is positive if the first string comes ^M
 * after the second.  The NSPR implementation is not i18n.  If the maximum^M
 * is zero, only the existance or non-existance (pointer is null) of the^M
 * strings is compared.^M
 */^M
^M
PR_EXTERN(PRIntn)^M
PL_strncmp(const char *a, const char *b, PRUint32 max);^M
^M
/*^M
 * PL_strcasecmp^M
 *^M
 * Returns an integer, the sign of which -- positive, zero or negative --^M
 * reflects the case-insensitive lexical sorting order of the two strings^M
 * indicated.  The result is positive if the first string comes after the ^M
 * second.  The NSPR implementation is not i18n.^M
 */^M
^M
PR_EXTERN(PRIntn)^M
PL_strcasecmp(const char *a, const char *b);^M
^M
/*^M
 * PL_strncasecmp^M
 *^M
 * Returns an integer, the sign of which -- positive, zero or negative --^M
 * reflects the case-insensitive lexical sorting order of the first n characters^M
 * of the two strings indicated.  The result is positive if the first string comes ^M
 * after the second.  The NSPR implementation is not i18n.^M
 */^M
^M
PR_EXTERN(PRIntn)^M
PL_strncasecmp(const char *a, const char *b, PRUint32 max);^M
^M
/*^M
 * PL_strchr^M
 *^M
 * Returns a pointer to the first instance of the specified character in the^M
 * provided string.  It returns null if the character is not found, or if the^M
 * provided string is null.  The character may be the null character.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strchr(const char *s, char c);^M
^M
/*^M
 * PL_strrchr^M
 *^M
 * Returns a pointer to the last instance of the specified character in the^M
 * provided string.  It returns null if the character is not found, or if the^M
 * provided string is null.  The character may be the null character.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strrchr(const char *s, char c);^M
^M
/*^M
 * PL_strnchr^M
 * ^M
 * Returns a pointer to the first instance of the specified character within the^M
 * first n characters of the provided string.  It returns null if the character^M
 * is not found, or if the provided string is null.  The character may be the^M
 * null character.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strnchr(const char *s, char c, PRUint32 n);^M
^M
/*^M
 * PL_strnrchr^M
 *^M
 * Returns a pointer to the last instance of the specified character within the^M
 * first n characters of the provided string.  It returns null if the character is^M
 * not found, or if the provided string is null.  The character may be the null^M
 * character.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strnrchr(const char *s, char c, PRUint32 n);^M
^M
/*^M
 * NOTE: Looking for strcasechr, strcaserchr, strncasechr, or strncaserchr?^M
 * Use strpbrk, strprbrk, strnpbrk or strnprbrk.^M
 */^M
^M
/*^M
 * PL_strpbrk^M
 *^M
 * Returns a pointer to the first instance in the first string of any character^M
 * (not including the terminating null character) of the second string.  It returns^M
 * null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strpbrk(const char *s, const char *list);^M
^M
/*^M
 * PL_strprbrk^M
 *^M
 * Returns a pointer to the last instance in the first string of any character^M
 * (not including the terminating null character) of the second string.  It returns^M
 * null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strprbrk(const char *s, const char *list);^M
^M
/*^M
 * PL_strnpbrk^M
 *^M
 * Returns a pointer to the first instance (within the first n characters) of any^M
 * character (not including the terminating null character) of the second string.^M
 * It returns null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strnpbrk(const char *s, const char *list, PRUint32 n);^M
^M
/*^M
 * PL_strnprbrk^M
 *^M
 * Returns a pointer to the last instance (within the first n characters) of any^M
 * character (not including the terminating null character) of the second string.^M
 * It returns null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strnprbrk(const char *s, const char *list, PRUint32 n);^M
^M
/*^M
 * PL_strstr^M
 *^M
 * Returns a pointer to the first instance of the little string within the^M
 * big one.  It returns null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strstr(const char *big, const char *little);^M
^M
/*^M
 * PL_strrstr^M
 *^M
 * Returns a pointer to the last instance of the little string within the big one.^M
 * It returns null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strrstr(const char *big, const char *little);^M
^M
/*^M
 * PL_strnstr^M
 *^M
 * Returns a pointer to the first instance of the little string within the first^M
 * n characters of the big one.  It returns null if either string is null.  It^M
 * returns null if the length of the little string is greater than n.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strnstr(const char *big, const char *little, PRUint32 n);^M
^M
/*^M
 * PL_strnrstr^M
 *^M
 * Returns a pointer to the last instance of the little string within the first^M
 * n characters of the big one.  It returns null if either string is null.  It^M
 * returns null if the length of the little string is greater than n.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strnrstr(const char *big, const char *little, PRUint32 max);^M
^M
/*^M
 * PL_strcasestr^M
 *^M
 * Returns a pointer to the first instance of the little string within the big one,^M
 * ignoring case.  It returns null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strcasestr(const char *big, const char *little);^M
^M
/*^M
 * PL_strcaserstr^M
 *^M
 * Returns a pointer to the last instance of the little string within the big one,^M
 * ignoring case.  It returns null if either string is null.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strcaserstr(const char *big, const char *little);^M
^M
/*^M
 * PL_strncasestr^M
 *^M
 * Returns a pointer to the first instance of the listtle string within the first^M
 * n characters of the big one, ignoring case.  It returns null if either string is ^M
 * null.  It returns null if the length of the little string is greater than n.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strncasestr(const char *big, const char *little, PRUint32 max);^M
^M
/*^M
 * PL_strncaserstr^M
 *^M
 * Returns a pointer to the last instance of the little string within the first^M
 * n characters of the big one, ignoring case.  It returns null if either string is^M
 * null.  It returns null if the length of the little string is greater than n.^M
 */^M
^M
PR_EXTERN(char *)^M
PL_strncaserstr(const char *big, const char *little, PRUint32 max);^M
^M
/*^M
 * Things not (yet?) included: strspn/strcspn, strtok/strtok_r, strsep.^M
 * memchr, memcmp, memcpy, memccpy, index, rindex, bcmp, bcopy, bzero.^M
 * Any and all i18n/l10n stuff.^M
 */^M
^M
PR_END_EXTERN_C^M
^M
#endif /* _plstr_h */^M
