/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
/*^M
** File:		prlock.h^M
** Description:	API to basic locking functions of NSPR.^M
**^M
**^M
** NSPR provides basic locking mechanisms for thread synchronization.  Locks ^M
** are lightweight resource contention controls that prevent multiple threads ^M
** from accessing something (code/data) simultaneously.^M
**/^M
^M
#ifndef prlock_h___^M
#define prlock_h___^M
^M
#include "prtypes.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/**********************************************************************/^M
/************************* TYPES AND CONSTANTS ************************/^M
/**********************************************************************/^M
^M
/*^M
 * PRLock --^M
 *^M
 *     NSPR represents the lock as an opaque entity to the client of the^M
 *	   API.  All routines operate on a pointer to this opaque entity.^M
 */^M
^M
typedef struct PRLock PRLock;^M
^M
/**********************************************************************/^M
/****************************** FUNCTIONS *****************************/^M
/**********************************************************************/^M
^M
/***********************************************************************^M
** FUNCTION:    PR_NewLock^M
** DESCRIPTION:^M
**  Returns a pointer to a newly created opaque lock object.^M
** INPUTS:      void^M
** OUTPUTS:     void^M
** RETURN:      PRLock*^M
**   If the lock can not be created because of resource constraints, NULL^M
**   is returned.^M
**  ^M
***********************************************************************/^M
NSPR_API(PRLock*) PR_NewLock(void);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_DestroyLock^M
** DESCRIPTION:^M
**  Destroys a given opaque lock object.^M
** INPUTS:      PRLock *lock^M
**              Lock to be freed.^M
** OUTPUTS:     void^M
** RETURN:      None^M
***********************************************************************/^M
NSPR_API(void) PR_DestroyLock(PRLock *lock);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_Lock^M
** DESCRIPTION:^M
**  Lock a lock.^M
** INPUTS:      PRLock *lock^M
**              Lock to locked.^M
** OUTPUTS:     void^M
** RETURN:      None^M
***********************************************************************/^M
NSPR_API(void) PR_Lock(PRLock *lock);^M
^M
/***********************************************************************^M
** FUNCTION:    PR_Unlock^M
** DESCRIPTION:^M
**  Unlock a lock.  Unlocking an unlocked lock has undefined results.^M
** INPUTS:      PRLock *lock^M
**              Lock to unlocked.^M
** OUTPUTS:     void^M
** RETURN:      PR_STATUS^M
**              Returns PR_FAILURE if the caller does not own the lock.^M
***********************************************************************/^M
NSPR_API(PRStatus) PR_Unlock(PRLock *lock);^M
^M
PR_END_EXTERN_C^M
^M
#endif /* prlock_h___ */^M
