/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */^M
/* ^M
 * The contents of this file are subject to the Mozilla Public^M
 * License Version 1.1 (the "License"); you may not use this file^M
 * except in compliance with the License. You may obtain a copy of^M
 * the License at http://www.mozilla.org/MPL/^M
 * ^M
 * Software distributed under the License is distributed on an "AS^M
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or^M
 * implied. See the License for the specific language governing^M
 * rights and limitations under the License.^M
 * ^M
 * The Original Code is the Netscape Portable Runtime (NSPR).^M
 * ^M
 * The Initial Developer of the Original Code is Netscape^M
 * Communications Corporation.  Portions created by Netscape are ^M
 * Copyright (C) 1998-2000 Netscape Communications Corporation.  All^M
 * Rights Reserved.^M
 * ^M
 * Contributor(s):^M
 * ^M
 * Alternatively, the contents of this file may be used under the^M
 * terms of the GNU General Public License Version 2 or later (the^M
 * "GPL"), in which case the provisions of the GPL are applicable ^M
 * instead of those above.  If you wish to allow use of your ^M
 * version of this file only under the terms of the GPL and not to^M
 * allow others to use your version of this file under the MPL,^M
 * indicate your decision by deleting the provisions above and^M
 * replace them with the notice and other provisions required by^M
 * the GPL.  If you do not delete the provisions above, a recipient^M
 * may use your version of this file under either the MPL or the^M
 * GPL.^M
 */^M
^M
#if defined(_PRMWAIT_H)^M
#else^M
#define _PRMWAIT_H^M
^M
#include "prio.h"^M
#include "prtypes.h"^M
#include "prclist.h"^M
^M
PR_BEGIN_EXTERN_C^M
^M
/********************************************************************************/^M
/********************************************************************************/^M
/********************************************************************************/^M
/******************************       WARNING        ****************************/^M
/********************************************************************************/^M
/**************************** This is work in progress. *************************/^M
/************************** Do not make any assumptions *************************/^M
/************************** about the stability of this *************************/^M
/************************** API or the underlying imple- ************************/^M
/************************** mentation.                   ************************/^M
/********************************************************************************/^M
/********************************************************************************/^M
^M
/*^M
** STRUCTURE:   PRWaitGroup^M
** DESCRIPTION:^M
**      The client may define several wait groups in order to semantically^M
**      tie a collection of file descriptors for a single purpose. This allows^M
**      easier dispatching of threads that returned with active file descriptors^M
**      from the wait function.^M
*/^M
typedef struct PRWaitGroup PRWaitGroup;^M
^M
/*^M
** ENUMERATION: PRMWStatus^M
** DESCRIPTION:^M
**      This enumeration is used to indicate the completion status of^M
**      a receive wait object. Generally stated, a positive value indicates^M
**      that the operation is not yet complete. A zero value indicates^M
**      success (similar to PR_SUCCESS) and any negative value is an^M
**      indication of failure. The reason for the failure can be retrieved^M
**      by calling PR_GetError().^M
**^M
**  PR_MW_PENDING       The operation is still pending. None of the other^M
**                      fields of the object are currently valid.^M
**  PR_MW_SUCCESS       The operation is complete and it was successful.^M
**  PR_MW_FAILURE       The operation failed. The reason for the failure^M
**                      can be retrieved by calling PR_GetError().^M
**  PR_MW_TIMEOUT       The amount of time allowed for by the object's^M
**                      'timeout' field has expired w/o the operation^M
**                      otherwise coming to closure.^M
**  PR_MW_INTERRUPT     The operation was cancelled, either by the client^M
**                      calling PR_CancelWaitFileDesc() or destroying the^M
**                      entire wait group (PR_DestroyWaitGroup()).^M
*/^M
typedef enum PRMWStatus^M
{^M
    PR_MW_PENDING = 1,^M
    PR_MW_SUCCESS = 0,^M
    PR_MW_FAILURE = -1,^M
    PR_MW_TIMEOUT = -2,^M
    PR_MW_INTERRUPT = -3^M
} PRMWStatus;^M
^M
/*^M
** STRUCTURE:   PRMemoryDescriptor^M
** DESCRIPTION:^M
**      THis is a descriptor for an interval of memory. It contains a^M
**      pointer to the first byte of that memory and the length (in^M
**      bytes) of the interval.^M
*/^M
typedef struct PRMemoryDescriptor^M
{^M
    void *start;                /* pointer to first byte of memory */^M
    PRSize length;              /* length (in bytes) of memory interval */^M
} PRMemoryDescriptor;^M
^M
/*^M
** STRUCTURE:   PRMWaitClientData^M
** DESCRIPTION:^M
**      An opague stucture for which a client MAY give provide a concrete^M
**      definition and associate with a receive descriptor. The NSPR runtime^M
**      does not manage this field. It is completely up to the client.^M
*/^M
typedef struct PRMWaitClientData PRMWaitClientData;^M
^M
/*^M
** STRUCTURE:   PRRecvWait^M
** DESCRIPTION:^M
**      A receive wait object contains the file descriptor that is subject^M
**      to the wait and the amount of time (beginning epoch established^M
**      when the object is presented to the runtime) the the channel should^M
**      block before abandoning the process.^M
**^M
**      The success of the wait operation will be noted in the object's^M
**      'outcome' field. The fields are not valid when the NSPR runtime^M
**      is in possession of the object.^M
**^M
**      The memory descriptor describes an interval of writable memory^M
**      in the caller's address space where data from an initial read^M
**      can be placed. The description may indicate a null interval.^M
*/^M
typedef struct PRRecvWait ^M
{^M
    PRCList internal;           /* internal runtime linkages */^M
^M
    PRFileDesc *fd;             /* file descriptor associated w/ object */^M
    PRMWStatus outcome;         /* outcome of the current/last operation */^M
    PRIntervalTime timeout;     /* time allowed for entire operation */^M
^M
    PRInt32 bytesRecv;          /* number of bytes transferred into buffer */^M
    PRMemoryDescriptor buffer;  /* where to store first segment of input data */^M
    PRMWaitClientData *client;  /* pointer to arbitrary client defined data */^M
} PRRecvWait;^M
^M
/*^M
** STRUCTURE:   PRMWaitEnumerator^M
** DESCRIPTION:^M
**      An enumeration object is used to store the state of an existing^M
**      enumeration over a wait group. The opaque object must be allocated^M
**      by the client and the reference presented on each call to the^M
**      pseudo-stateless enumerator. The enumeration objects are sharable^M
**      only in serial fashion.^M
*/^M
typedef struct PRMWaitEnumerator PRMWaitEnumerator;^M
^M
^M
/*^M
** FUNCTION:    PR_AddWaitFileDesc^M
** DESCRIPTION:^M
**      This function will effectively add a file descriptor to the^M
**      list of those waiting for network receive. The new descriptor^M
**      will be semantically tied to the wait group specified.^M
**^M
**      The ownership for the storage pointed to by 'desc' is temporarily^M
**      passed over the the NSPR runtime. It will be handed back by the^M
**      function PR_WaitRecvReady().^M
**^M
**  INPUTS^M
**      group       A reference to a PRWaitGroup or NULL. Wait groups are^M
**                  created by calling PR_CreateWaitGroup() and are used^M
**                  to semantically group various file descriptors by the^M
**                  client's application.^M
**      desc        A reference to a valid PRRecvWait. The object of the^M
**                  reference must be preserved and not be modified^M
**                  until its ownership is returned to the client.^M
**  RETURN^M
**      PRStatus    An indication of success. If equal to PR_FAILUE details^M
**                  of the failure are avaiable via PR_GetError().^M
**^M
**  ERRORS^M
**      PR_INVALID_ARGUMENT_ERROR^M
**                  Invalid 'group' identifier or duplicate 'desc' object.^M
**      PR_OUT_OF_MEMORY_ERROR^M
**                  Insuffient memory for internal data structures.^M
**      PR_INVALID_STATE_ERROR^M
**                  The group is being destroyed.^M
*/^M
NSPR_API(PRStatus) PR_AddWaitFileDesc(PRWaitGroup *group, PRRecvWait *desc);^M
^M
/*^M
** FUNCTION:    PR_WaitRecvReady^M
** DESCRIPTION:^M
**      PR_WaitRecvReady will block the calling thread until one of the^M
**      file descriptors that have been added via PR_AddWaitFileDesc is^M
**      available for input I/O.^M
**  INPUT^M
**      group       A pointer to a valid PRWaitGroup or NULL (the null^M
**                  group. The function will block the caller until a^M
**                  channel from the wait group becomes ready for receive^M
**                  or there is some sort of error.^M
**  RETURN^M
**      PRReciveWait^M
**                  When the caller is resumed it is either returned a^M
**                  valid pointer to a previously added receive wait or^M
**                  a NULL. If the latter, the function has terminated^M
**                  for a reason that can be determined by calling^M
**                  PR_GetError().^M
**                  If a valid pointer is returned, the reference is to the^M
**                  file descriptor contained in the receive wait object.^M
**                  The outcome of the wait operation may still fail, and^M
**                  if it has, that fact will be noted in the object's^M
**                  outcome field. Details can be retrieved from PR_GetError().^M
**^M
**  ERRORS^M
**      PR_INVALID_ARGUMENT_ERROR^M
**                  The 'group' is not known by the runtime.^M
**      PR_PENDING_INTERRUPT_ERROR^M
                    The thread was interrupted.^M
**      PR_INVALID_STATE_ERROR^M
**                  The group is being destroyed.^M
*/^M
NSPR_API(PRRecvWait*) PR_WaitRecvReady(PRWaitGroup *group);^M
^M
/*^M
** FUNCTION:    PR_CancelWaitFileDesc^M
** DESCRIPTION:^M
**      PR_CancelWaitFileDesc is provided as a means for cancelling operations^M
**      on objects previously submitted by use of PR_AddWaitFileDesc(). If^M
**      the runtime knows of the object, it will be marked as having failed^M
**      because it was interrupted (similar to PR_Interrupt()). The first^M
**      available thread waiting on the group will be made to return the^M
**      PRRecvWait object with the outcome noted.^M
**^M
**  INPUTS^M
**      group       The wait group under which the wait receive object was^M
**                  added.^M
**      desc        A pointer to the wait receive object that is to be^M
**                  cancelled.^M
**  RETURN^M
**      PRStatus    If the wait receive object was located and associated^M
**                  with the specified wait group, the status returned will^M
**                  be PR_SUCCESS. There is still a race condition that would^M
**                  permit the offected object to complete normally, but it^M
**                  is assured that it will complete in the near future.^M
**                  If the receive object or wait group are invalid, the^M
**                  function will return with a status of PR_FAILURE.^M
**^M
**  ERRORS^M
**      PR_INVALID_ARGUMENT_ERROR^M
**                  The 'group' argument is not recognized as a valid group.^M
**      PR_COLLECTION_EMPTY_ERROR^M
**                  There are no more receive wait objects in the group's^M
**                  collection.^M
**      PR_INVALID_STATE_ERROR^M
**                  The group is being destroyed.^M
*/^M
NSPR_API(PRStatus) PR_CancelWaitFileDesc(PRWaitGroup *group, PRRecvWait *desc);^M
^M
/*^M
** FUNCTION:    PR_CancelWaitGroup^M
** DESCRIPTION:^M
**      PR_CancelWaitGroup is provided as a means for cancelling operations^M
**      on objects previously submitted by use of PR_AddWaitFileDesc(). Each^M
**      successive call will return a pointer to a PRRecvWait object that^M
**      was previously registered via PR_AddWaitFileDesc(). If no wait^M
**      objects are associated with the wait group, a NULL will be returned.^M
**      This function should be called in a loop until a NULL is returned^M
**      to reclaim all the wait objects prior to calling PR_DestroyWaitGroup().^M
**^M
**  INPUTS^M
**      group       The wait group under which the wait receive object was^M
**                  added.^M
**  RETURN^M
**      PRRecvWait* If the wait group is valid and at least one receive wait^M
**                  object is present in the group, that object will be^M
**                  marked as PR_MW_INTERRUPT'd and removed from the group's^M
**                  queues. Otherwise a NULL will be returned and the reason^M
**                  for the NULL may be retrieved by calling PR_GetError().^M
**^M
**  ERRORS^M
**      PR_INVALID_ARGUMENT_ERROR^M
**      PR_GROUP_EMPTY_ERROR^M
*/^M
NSPR_API(PRRecvWait*) PR_CancelWaitGroup(PRWaitGroup *group);^M
^M
/*^M
** FUNCTION:    PR_CreateWaitGroup^M
** DESCRIPTION:^M
**      A wait group is an opaque object that a client may create in order^M
**      to semantically group various wait requests. Each wait group is^M
**      unique, including the default wait group (NULL). A wait request^M
**      that was added under a wait group will only be serviced by a caller^M
**      that specified the same wait group.^M
**^M
**  INPUT^M
**      size        The size of the hash table to be used to contain the^M
**                  receive wait objects. This is just the initial size.^M
**                  It will grow as it needs to, but to avoid that hassle^M
**                  one can suggest a suitable size initially. It should^M
**                  be ~30% larger than the maximum number of receive wait^M
**                  objects expected.^M
**  RETURN^M
**      PRWaitGroup If successful, the function will return a pointer to an^M
**                  object that was allocated by and owned by the runtime.^M
**                  The reference remains valid until it is explicitly destroyed^M
**                  by calling PR_DestroyWaitGroup().^M
**^M
**  ERRORS^M
**      PR_OUT_OF_MEMORY_ERROR^M
*/^M
NSPR_API(PRWaitGroup*) PR_CreateWaitGroup(PRInt32 size);^M
^M
/*^M
** FUNCTION:    PR_DestroyWaitGroup^M
** DESCRIPTION:^M
**      Undo the effects of PR_CreateWaitGroup(). Any receive wait operations^M
**      on the group will be treated as if the each had been the target of a^M
**      PR_CancelWaitFileDesc().^M
**^M
**  INPUT^M
**      group       Reference to a wait group previously allocated using^M
**                  PR_CreateWaitGroup().^M
**  RETURN^M
**      PRStatus    Will be PR_SUCCESS if the wait group was valid and there^M
**                  are no receive wait objects in that group. Otherwise^M
**                  will indicate PR_FAILURE.^M
**^M
**  ERRORS^M
**      PR_INVALID_ARGUMENT_ERROR^M
**                  The 'group' argument does not reference a known object.^M
**      PR_INVALID_STATE_ERROR^M
**                  The group still contains receive wait objects.^M
*/^M
NSPR_API(PRStatus) PR_DestroyWaitGroup(PRWaitGroup *group);^M
^M
/*^M
** FUNCTION:    PR_CreateMWaitEnumerator^M
** DESCRIPTION:^M
**      The PR_CreateMWaitEnumerator() function returns a reference to an^M
**      opaque PRMWaitEnumerator object. The enumerator object is required^M
**      as an argument for each successive call in the stateless enumeration^M
**      of the indicated wait group.^M
**^M
**      group       The wait group that the enumeration is intended to^M
**                  process. It may be be the default wait group (NULL).^M
** RETURN^M
**      PRMWaitEnumerator* group^M
**                  A reference to an object that will be used to store^M
**                  intermediate state of enumerations.^M
** ERRORS^M
**      Errors are indicated by the function returning a NULL.^M
**      PR_INVALID_ARGUMENT_ERROR^M
**                  The 'group' argument does not reference a known object.^M
**      PR_OUT_OF_MEMORY_ERROR^M
*/^M
NSPR_API(PRMWaitEnumerator*) PR_CreateMWaitEnumerator(PRWaitGroup *group);^M
^M
/*^M
** FUNCTION:    PR_DestroyMWaitEnumerator^M
** DESCRIPTION:^M
**      Destroys the object created by PR_CreateMWaitEnumerator(). The reference^M
**      used as an argument becomes invalid.^M
**^M
** INPUT^M
**      PRMWaitEnumerator* enumerator^M
**          The PRMWaitEnumerator object to destroy.^M
** RETURN^M
**      PRStatus^M
**          PR_SUCCESS if successful, PR_FAILURE otherwise.^M
** ERRORS^M
**      PR_INVALID_ARGUMENT_ERROR^M
**                  The enumerator is invalid.^M
*/^M
NSPR_API(PRStatus) PR_DestroyMWaitEnumerator(PRMWaitEnumerator* enumerator);^M
^M
/*^M
** FUNCTION:    PR_EnumerateWaitGroup^M
** DESCRIPTION:^M
**      PR_EnumerateWaitGroup is a thread safe enumerator over a wait group.^M
**      Each call to the enumerator must present a valid PRMWaitEnumerator^M
**      rererence and a pointer to the "previous" element returned from the^M
**      enumeration process or a NULL.^M
**^M
**      An enumeration is started by passing a NULL as the "previous" value.^M
**      Subsequent calls to the enumerator must pass in the result of the^M
**      previous call. The enumeration end is signaled by the runtime returning^M
**      a NULL as the result.^M
**^M
**      Modifications to the content of the wait group are allowed during^M
**      an enumeration. The effect is that the enumeration may have to be^M
**      "reset" and that may result in duplicates being returned from the^M
**      enumeration.^M
**^M
**      An enumeration may be abandoned at any time. The runtime is not^M
**      keeping any state, so there are no issues in that regard.^M
*/^M
NSPR_API(PRRecvWait*) PR_EnumerateWaitGroup(^M
    PRMWaitEnumerator *enumerator, const PRRecvWait *previous);^M
   ^M
PR_END_EXTERN_C^M
^M
#endif /* defined(_PRMWAIT_H) */^M
^M
/* prmwait.h */^M
